/*
 * MIT License
 *
 * Copyright (c) 2019 norangebit
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package it.norangeb.algorithms.graph

import it.norangeb.algorithms.graph.data.EdgeI
import it.norangeb.algorithms.graph.data.WeightAdjacentList
import it.norangeb.algorithms.graph.data.WeightGraphData

class WeightDiGraph(
    private val data: WeightGraphData = WeightAdjacentList()
) : WeightDirectedGraph {
    override fun vertexNumber(): Int = data.vertexNumber()

    override fun edgeNumber(): Int = data.edgeNumber()

    override fun addEdge(edge: EdgeI) = data.addEdge(edge)

    override fun adjacentVertex(vertex: Int): Collection<EdgeI> = data.adjacent(vertex)

    override fun reverse(): WeightDirectedGraph {
        val reverse = WeightDiGraph()

        (0 until data.vertexNumber())
            .forEach { from ->
                data.adjacent(from).forEach {
                    reverse.addEdge(it.reversed())
                }
            }

        return reverse
    }

    override fun edges(): Collection<EdgeI> {
        val edges = mutableListOf<EdgeI>()

        (0 until data.vertexNumber())
            .forEach {
                edges.addAll(adjacentVertex(it))
            }

        return edges
    }
}